@@ -10,7 +10,7 @@
 # http://www.wtfpl.net/ for more details.
 
 package Chess::Plisco::EPD;
-$Chess::Plisco::EPD::VERSION = '0.6';
+$Chess::Plisco::EPD::VERSION = 'v1.0.0';
 use strict;
 use integer;
 
@@ -20,25 +20,32 @@ use Locale::TextDomain qw('Chess-Plisco');
 use Chess::Plisco::EPD::Record;
 
 sub new {
-	my ($class, $arg, $filename) = @_;
+	my ($class, $arg, @args) = @_;
+
+	my %options = @args;
+	if (@args == 1 && 'HASH' eq ref $args[0]) {
+		%options = %{$args[0]};
+	}
+	my $filename = $options{filename};
+	my $pseudo_legal = $options{pseudo_legal};
 
 	my $reftype = (reftype $arg) || '';
 	my @lines;
 	if ('SCALAR' eq $reftype) {
 		@lines = split /\n/, $$arg;
-		$filename = __"[in-memory string]";
+		$filename //= __"[in-memory string]";
 	} elsif ('ARRAY' eq $reftype) {
 		@lines = @$arg;
-		$filename = __"[in-memory array]";
+		$filename //= __"[in-memory array]";
 	} elsif ('GLOB' eq $reftype) {
 		@lines = <$arg>;
-		$filename ||= __"[file-handle]";
+		$filename //= __"[file-handle]";
 	} else {
 		open my $fh, '<', $arg
 			or die __x("cannot open '{filename}' for reading: {error}!\n",
 				filename => $arg,
 				error => $!);
-		$filename ||= $arg;
+		$filename //= $arg;
 		@lines = <$fh>;
 	}
 
@@ -51,7 +58,7 @@ sub new {
 		$line =~ s/ws+$//;
 		next if !length $line;
 
-		my $record = eval { Chess::Plisco::EPD::Record->new($line) };
+		my $record = eval { Chess::Plisco::EPD::Record->new($line, $pseudo_legal) };
 		if ($@) {
 			die "$filename:$lineno: $@";
 		}
