@@ -1,6 +1,6 @@
 #! /bin/false
 
-# Copyright (C) 2021 Guido Flohr <guido.flohr@cantanea.com>,
+# Copyright (C) 2021-2025 Guido Flohr <guido.flohr@cantanea.com>,
 # all rights reserved.
 
 # This program is free software. It comes without any warranty, to
@@ -14,8 +14,8 @@
 ## no critic (TestingAndDebugging::RequireUseStrict)
 
 (do {
-	my $her_color = !$c;
-	my $her_pieces = $p->[CP_POS_WHITE_PIECES + $her_color];
+	my $her_colour = !$c;
+	my $her_pieces = $p->[CP_POS_WHITE_PIECES + $her_colour];
 	my $occupancy = $p->[CP_POS_WHITE_PIECES + $c] | $her_pieces;
 	my $queens = cp_pos_queens($p);
 	$her_pieces
