@@ -1,6 +1,6 @@
 #! /bin/false
 
-# Copyright (C) 2021 Guido Flohr <guido.flohr@cantanea.com>,
+# Copyright (C) 2021-2025 Guido Flohr <guido.flohr@cantanea.com>,
 # all rights reserved.
 
 # This program is free software. It comes without any warranty, to
@@ -14,6 +14,7 @@
 ## no critic (TestingAndDebugging::RequireUseStrict)
 
 while ($target_mask) {
-	push @m, $b | cp_bitboard_count_trailing_zbits $t;
+	my $to = cp_bitboard_count_trailing_zbits $t;
+	push @m, $b | ($to << 15) | ($board[$to] << 3);
 	$target_mask = cp_bitboard_clear_least_set $target_mask;
 }
